#!/usr/bin/perl
#*******************************************************************#
#  iPKGui - 3rdparty ipkg/opkg browser/webUI                        #
#  Modul:       config.cgi                                          #
#  Description: a perl script to read/write data from a ajax call   #
#  Author:      QTip from the german Synology support forum         #
#  Copyright:   2013-2015 by QTip                                   #
#  License:     GNU GPLv3 (see LICENSE)                             #
#  ---------------------------------------------------------------  #
#  Version:     0.25 - 26/07/2015                                   #
#  for more information check the changelog                         #
#*******************************************************************#

$user = `/usr/syno/synoman/webman/modules/authenticate.cgi`;
$user =~ s/^\s+|\s+$//g;
exit if ($user eq '');

use strict;
use warnings;
use lib ('/var/packages/ipkgui/target/include','/var/packages/ipkgui/target/lib');

use CGI;
use CGI::Carp qw(fatalsToBrowser);
use JSON::XS;
use File::Copy;
use Cwd qw(realpath);
use Fcntl qw(:flock);
#use Data::Dumper;

## initialize vars #################
my (@parts,$part,$buffer,$value,$name,%param,$raw_data,$feedurl,@json_data,$cron_entry,@cron_data,%match,$result);
my $packagehome = "/var/packages/ipkgui";
my $configpath = $packagehome."/etc/";
my $pmconfigfile = $configpath."pmconfig.json";
my $pmtemplatefile = "$packagehome/target/extras/pmtemplates.json";
my $archivehtaccess = "$packagehome/target/extras/htaccess";
my $crontab = "/etc/crontab";
my $gzipcmd = "/bin/gzip";

require "functions.pl";
###################################

print "Content-type: application/json; charset=UTF-8\n\n";

if ($ENV{'REQUEST_METHOD'} eq "POST") {
     read(STDIN,$buffer,$ENV{"CONTENT_LENGTH"});
     @parts = split(/\&/,$buffer);
     foreach $part (@parts) {
          ($name,$value) = split(/=/,$part);
          $value =~ tr/+/ /;
          $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
          # prevent include of server side includes
          $value =~ s/<!--(.|\n)*-->//g;
          $param{$name} = $value;
     }
     my $pmconfig = detect_pkm($pmconfigfile,$pmtemplatefile);
     $pmconfig->{'feedisdir'} = (-d $pmconfig->{'feeds'} ? do {bless \(my $dummy = 1), "JSON::XS::Boolean"} : do {bless \(my $dummy = 0), "JSON::XS::Boolean"});
     if ($param{'action'} eq "read" && $param{'file'} ne "") {
          if ($param{'file'} eq "pmconfig") {
               $raw_data = JSON::XS->new->encode($pmconfig);
               print '{"'.$param{'file'}.'":['.$raw_data.']}';
               exit;
          } else {
               my $config_file = $configpath.$param{'file'}.".json";
               if (-e $config_file) {
                    if (open(DAT, $config_file)) {
                         flock(DAT, LOCK_EX);
                         $raw_data = <DAT>;
                         close(DAT);
                    } else {
                         close(DAT);
                         print '{"success":false}';
                         exit;
                    }
               } else {
                    print '{"'.$param{'file'}.'":[]}';
                    exit;
               }
               print $raw_data;
          }
     } elsif ($param{'action'} eq "write" && $param{'file'} ne "" && $param{'value'} ne "") {
          if ($param{'file'} eq "options") {
               @json_data = JSON::XS->new->decode($param{'value'});
               my $archive_feed = $json_data[0]{'options'}[0]{'archive_feed'};
               my $archive_public = $json_data[0]{'options'}[0]{'archive_public'};
               my $ibarchivefile = $pmconfig->{'listpath'}."/archive";
               if ($archive_feed eq "1") {
                    if (-f $ibarchivefile && ! -f $pmconfig->{'archivepath'}."/Packages.gz") {
                         $result = `$gzipcmd -c $ibarchivefile > $pmconfig->{'archivepath'}/Packages.gz`;
                    }
               } else {
                    if (-e $pmconfig->{'archivepath'}."/Packages.gz") {
                         unlink($pmconfig->{'archivepath'}."/Packages.gz");
                    }
               }
               if ($archive_public eq "1") {
                    if (-f $archivehtaccess && ! -f $pmconfig->{'archivepath'}."/.htaccess") {
                         copy($archivehtaccess, $pmconfig->{'archivepath'}."/.htaccess");
                         chown(0,0,$pmconfig->{'archivepath'}."/.htaccess");
                         chmod(0644,$pmconfig->{'archivepath'}."/.htaccess");
                    }
               } else {
                    if (-f $pmconfig->{'archivepath'}."/.htaccess") {
                         unlink($pmconfig->{'archivepath'}."/.htaccess");
                    }
               }
               if ($json_data[0]{'options'}[0]{'check_enabled'} == 1) {
                    # enable or change cron
                    my $check_interval = $json_data[0]{'options'}[0]{'check_interval'};
                    my $check_time = $json_data[0]{'options'}[0]{'check_time'};
                    my ($cron_hour,$cron_minute) = split(":",$check_time);
                    if (substr($check_interval,0,1) eq "d") {
                         $cron_entry = int($cron_minute)."	".int($cron_hour)."	*/".substr($check_interval,1,1)."	*	*	root	".$packagehome."/target/include/cron.pl 2>/dev/null\n";
                    } elsif ($check_interval eq "w") {
                         $cron_entry = int($cron_minute)."	".int($cron_hour)."	*/7	*	*	root	".$packagehome."/target/include/cron.pl 2>/dev/null\n";
                    } elsif ($check_interval eq "m") {
                         $cron_entry = int($cron_minute)."	".int($cron_hour)."	*	*/1	*	root	".$packagehome."/target/include/cron.pl 2>/dev/null\n";
                    }
                    @cron_data = read_crontab($crontab,"$packagehome/target/include/cron.pl 2>/dev/null");
                    if (@cron_data) {
                         push @cron_data, $cron_entry;
                         my $ret = write_crontab($crontab,@cron_data);
                         if ($ret != 0) {
                              print '{"success":false}';
                              exit;
                         }
                    } else {
                         print '{"success":false}';
                         exit;
                    }
               } else {
                    # disable cron
                    @cron_data = read_crontab($crontab,"$packagehome/target/include/cron.pl 2>/dev/null");
                    if (@cron_data) {
                         my $ret = write_crontab($crontab,@cron_data);
                         if ($ret != 0) {
                              print '{"success":false}';
                              exit;
                         }
                    } else {
                         print '{"success":false}';
                         exit;
                    }
               }
               # send signal to force crond to re-read the crontab
               system('/bin/kill -1 $(/bin/cat /var/run/crond.pid)');
          } elsif ($param{'file'} eq "pmconfig") {
               @json_data = JSON::XS->new->decode($param{'value'});
               my $old_archivepath = realpath($pmconfig->{'archivepath'});
               my $new_archivepath = realpath($json_data[0]{'pmconfig'}[0]{'archivepath'});
               my $old_sym_target = $pmconfig->{'archiveurl'};
               my $new_sym_target = $json_data[0]{'pmconfig'}[0]{'archiveurl'};
#print "old path: ".$old_archivepath." new path: ".$new_archivepath."\n";
               if ($new_archivepath ne $old_archivepath || $new_sym_target ne $old_sym_target) {
                    my $pkg_tp = '/usr/syno/synoman/webman/3rdparty/ipkgui';
                    my $pkg_vol = realpath($pkg_tp);
                    if ($old_sym_target =~ m/(.*)(webman.*)/i) {
                         $old_sym_target =~ s/^(.*)(webman.*)/\/usr\/syno\/synoman\/$2/i;
                    } else {
                         $old_sym_target =~ s/^((https?):\/\/(www\.)?[a-z0-9\-:@]+(\.?[a-z0-9\-:@]+)+([\:0-9]+)?([\/?].*)?)/\/var\/services\/web$6/i;
                    }
#print "old url: $old_sym_target\n";
                    if ($new_sym_target =~ m/(.*)(webman.*)/i) {
                         $new_sym_target =~ s/^(.*)(webman.*)/\/usr\/syno\/synoman\/$2/i;
                    } else {
                         $new_sym_target =~ s/^((https?):\/\/(www\.)?[a-z0-9\-:@]+(\.?[a-z0-9\-:@]+)+([\:0-9]+)?([\/?].*)?)/\/var\/services\/web$6/i;
                    }
#print "new url: $new_sym_target\n";
                    my $old_rest_url = substr($old_sym_target,length($pkg_tp));
                    my $old_sym_target_real = $pkg_vol.$old_rest_url;
                    my $new_rest_url = substr($new_sym_target,length($pkg_tp));
                    my $new_sym_target_real = $pkg_vol.$new_rest_url;
#print "old real: $old_sym_target_real\n";
#print "new real: $new_sym_target_real\n";
                    if ($new_archivepath ne $new_sym_target_real && -d $new_sym_target_real && ! -l $new_sym_target_real) {
                         print '{"success":false,"msg":"StatusWritePMCFolderText"}';
                         exit;
                    } elsif (-l $new_archivepath) {
                         print '{"success":false,"msg":"StatusWritePMCLinkText"}';
                         exit;
                    } elsif ($new_sym_target_real ne $new_archivepath) {
#print "$new_sym_target_real symlink anlegen\n";
                         unlink($old_sym_target_real) if (-l $old_sym_target && $old_sym_target_real eq $new_sym_target_real);
                         symlink($new_archivepath,$new_sym_target_real);
                    }
                    if (-l $old_sym_target && $old_sym_target_real ne $new_sym_target_real) {
#print "$old_sym_target_real lschen\n";
                         unlink($old_sym_target_real);
                    }
               }
          }

          open(DATA,">".$configpath.$param{'file'}.".json");
          flock(DATA, LOCK_EX);
          print DATA $param{'value'};
          close (DATA);
          if ($? == 0) {
               print '{"success":true}';
          } else {
               print '{"success":false}';
          }
     }
}
exit;
