#!/usr/bin/perl
#*******************************************************************#
#  iPKGui - 3rdparty ipkg/opkg browser/webUI                        #
#  Modul:       cron.pl                                             #
#  Description: a perl script for cron update checks                #
#  Author:      QTip from the german Synology support forum         #
#  Copyright:   2012-2015 by QTip                                   #
#  License:     GNU GPLv3 (see LICENSE)                             #
#  ---------------------------------------------------------------  #
#  Version:     0.4 - 17/07/2015                                    #
#  for more information check the changelog                         #
#*******************************************************************#

use strict;
use warnings;
use lib ('/var/packages/ipkgui/target/include','/var/packages/ipkgui/target/lib');

use JSON::XS;
use Encode qw(encode_utf8);

## initialize vars #################
my $packagename = "ipkgui";
my $packagehome = "/var/packages/$packagename";
my $configpath = $packagehome."/etc/";
my $pmconfigfile = $configpath."pmconfig.json";
my $pmtemplatefile = "$packagehome/target/extras/pmtemplates.json";
my $notifycommand = "/usr/syno/bin/synodsmnotify";
my $ibconfigfile = $configpath."options.json";
my $dsmmajor = `/bin/get_key_value /etc.defaults/VERSION majorversion`;
my $className = "SYNO.SDS._ThirdParty.App.".$packagename;
my ($pack_data,$raw_data,$json_data,$jdata,$notifylang);
my (@newpackages,@upgradepackages,$oldupgrades_size,$upgrades_size,%match,$notify_new,$notify_upgrades,$notify_appbadge);
my @packcron = ();

$ENV{'PATH'} = "/opt/bin:/opt/sbin:".$ENV{'PATH'};

require "functions.pl";

###################################

my $notifymessages = {
     'ger' => {
          'new'     => {'title' => 'iPKGui: Neue Pakete','message' => 'neue Pakete verfgbar'},
          'upgrade' => {'title' => 'iPKGui: Aktualisierungen','message' => 'Paketaktualisierungen verfgbar'},
          'error' => {'title' => 'iPKGui: Fehler','message' => 'Kein Paketmanager installiert!'},
          'footer'  => "&nbsp;&nbsp;<p><a style=font-size:12px\;text-decoration:underline\; href=javascript:SYNO.SDS.AppLaunch('".$className."',{fn:'".$className."'})>Hier klicken, um iPKGui zu starten.</a></p>"
     },
     'enu' => {
          'new'     => {'title' => 'iPKGui: New packages','message' => 'new Packages available'},
          'upgrade' => {'title' => 'iPKGui: Upgrades','message' => 'Package upgrades available'},
          'error' => {'title' => 'iPKGui: Error','message' => 'No package manager installed!'},
          'footer'  => "&nbsp;&nbsp;<p><a style=font-size:12px\;text-decoration:underline\; href=javascript:SYNO.SDS.AppLaunch('".$className."',{fn:'".$className."'})>Click here to launch iPKGui.</a></p>"
     },
     'fre' => {
          'new'     => {'title' => 'iPKGui: Nouveaux Paquets','message' => 'nouveaux paquets disponibles'},
          'upgrade' => {'title' => 'iPKGui: Mises  jour','message' => 'Mises  jour de paquets disponibles'},
          'error' => {'title' => 'iPKGui: Erreur','message' => 'Aucun gestionnaire de paquets install !'},
          'footer'  => "&nbsp;&nbsp;<p><a style=font-size:12px\;text-decoration:underline\; href=javascript:SYNO.SDS.AppLaunch('".$className."',{fn:'".$className."'})>Cliquez ici pour lancer iPKGui.</a></p>"
     }
};

sub all_user {
     my $type = shift;

     my (@raw_data1,@user,%user_);
     my $usercommand = "/usr/syno/sbin/synouser --enum ".$type." | /bin/grep -v 'guest' | /bin/grep -v 'User Listed:' 2> /dev/null|";
     if (open(DAT, $usercommand)) {
          @raw_data1 = <DAT>;
          close(DAT);
     } else {
          close(DAT);
     }
     if (@raw_data1) {
          foreach my $user_entry (@raw_data1) {
               chomp($user_entry);
               push @user, $user_entry;
          }
     }
     %user_ = map{$_,1}@user;
     return (keys %user_);
}
sub build_member {
     my $synouser = shift;
     my ($rec,@raw_data2,@member,%member_);
     foreach $rec (@$synouser) {
          if (substr($rec,0,1) eq '@') {
               if (substr($rec,1) eq 'users') {
                    push @member, all_user('local');
               } else {
                    my $groupmembercommand = "/usr/syno/sbin/synogroup --get ".substr($rec,1)." | /usr/bin/tail -n +5 | /usr/bin/awk -F'[' '{ print \$2 }' | /usr/bin/awk -F']' '{ print \$1 }' 2> /dev/null|";
                    if (open(DAT, $groupmembercommand)) {
                         @raw_data2 = <DAT>;
                         close(DAT);
                    } else {
                         close(DAT);
                    }
                    if (@raw_data2) {
                         foreach my $groupmember_entry (@raw_data2) {
                              chomp($groupmember_entry);
                              push @member, $groupmember_entry;
                         }
                    }
               }
          } else {
               push @member, $rec;
          }
     }
     %member_ = map{$_,1}@member;
     return (keys %member_);
}

sub send_dsmnotify {
     # send dsm notification
     my ($type,$count) = @_;
     my ($tagName,$r,$message,$title);
     my @notifyuser = split(",","\@administrators");

     # if tagname not empty, classname must have also a valid value
     # classname are in most cases the dom application name eg SYNO.SDS._ThirdParty.App.growler <- packagename
     # if classname empty, then the exclamation mark is used otherwise the application icon is used
     # known tagnames:
     # $tagName = "ok";
     # $tagName = "err";
     # $tagName = "true";
     # $tagName = "false";
     $tagName = "";

     my @groupmember = build_member(\@notifyuser);
     foreach $r (@groupmember) {
          $r = encode_utf8($r);
          $notifylang = `/bin/get_key_value /etc/synoinfo.conf maillang`;
          chomp($notifylang);
          $notifylang = ($notifylang ne "ger") ? "enu" : $notifylang;
          $title = $notifymessages->{$notifylang}->{$type}->{'title'};
          $message = $count." ".$notifymessages->{$notifylang}->{$type}->{'message'}.$notifymessages->{$notifylang}->{'footer'};
          $message =~ s/\n/<br>/g;
          $message = encode_utf8($message);
          my $ret = `$notifycommand -c "$className" -t "$tagName" "$r" "$title" "$message" >&1`;
     }
     if ($? != 0) {
          return 1;
     } else {
          return 0;
     }
}

# update list of packages
my $pmconfig = detect_pkm($pmconfigfile,$pmtemplatefile);
if (scalar(keys(%{$pmconfig})) == 0) {
     my $rt = send_dsmnotify("error",'');
     if ($rt != 0) {
          exit 1;
     }
     print '({"success":false,"msg":"StatusNoIPKGText"})';
     exit;
}
my $pmupdcmd = "$pmconfig->{'pmexec'} update 2> /dev/null";
my $pminfocmd = "$pmconfig->{'pmexec'} info 2> /dev/null|";
my $pmstatcmd = "$pmconfig->{'pmexec'} status 2> /dev/null|";
my @config_data = read_config($ibconfigfile);
if (exists($config_data[0]{'options'}[0])) {
     $notify_new = $config_data[0]{'options'}[0]{'notify_new'} if defined $config_data[0]{'options'}[0]{'notify_new'};
     $notify_upgrades = $config_data[0]{'options'}[0]{'notify_upgrades'} if defined $config_data[0]{'options'}[0]{'notify_upgrades'};
     $notify_appbadge = $config_data[0]{'options'}[0]{'notify_appbadge'} if defined $config_data[0]{'options'}[0]{'notify_appbadge'};
}
$oldupgrades_size = -s $configpath."upgrades.dat";
$oldupgrades_size = 0 if (!defined $oldupgrades_size);

my $ret = `$pmupdcmd`;
if ($? != 0) {
     exit 1;
}
require "$packagehome/target/include/package_read.pl";
require "$packagehome/target/include/package_update.pl";
@packcron = get_package_updates($pmstatcmd,$pminfocmd,$packagehome,$pmconfig->{'listpath'});

# save updated list as new reference
$pack_data = JSON::XS->new->utf8->encode(\@packcron);
if (open(DAT,">".$configpath."packages_ref.json")) {
     print DAT $pack_data;
     close (DAT);
} else {
     close(DAT);
}
if (open(DAT,"<".$configpath."new.dat")) {
     @newpackages = <DAT>;
     close (DAT);
} else {
     close(DAT);
}

if (open(DAT,"<".$configpath."upgrades.dat")) {
     @upgradepackages = <DAT>;
     close (DAT);
} else {
     close(DAT);
}

# send notify about new and upgrade packages if available and configured
if ($dsmmajor >= 3) {
     if (scalar(@newpackages) && $notify_new) {
          $ret = send_dsmnotify("new",scalar(@newpackages));
          if ($ret != 0) {
               exit 1;
          }
     }
     $upgrades_size = -s $configpath."upgrades.dat";
     $upgrades_size = 0 if (!defined $upgrades_size);
     if ($upgrades_size != $oldupgrades_size) {
          if (scalar(@upgradepackages) && $notify_upgrades) {
               $ret = send_dsmnotify("upgrade",scalar(@upgradepackages));
               if ($ret != 0) {
                    exit 1;
               }
          }
     }
     my $updatesum = scalar(@newpackages) + scalar(@upgradepackages);
     update_appnotify($className,$updatesum) if ($dsmmajor >= 5 && $notify_appbadge);
}

