#!/usr/bin/perl
#*******************************************************************#
#  iPKGui - 3rdparty ipkg/opkg browser/webUI                        #
#  Modul:       execute.cgi                                         #
#  Description: a perl script to execute ipkg commands              #
#  Author:      QTip from the german Synology support forum         #
#  Copyright:   2012-2015 by QTip                                   #
#  License:     GNU GPLv3 (see LICENSE)                             #
#  ---------------------------------------------------------------  #
#  Version:     0.66 - 21/10/2015                                   #
#  for more information check the changelog                         #
#*******************************************************************#

$user = `/usr/syno/synoman/webman/modules/authenticate.cgi`;
$user =~ s/^\s+|\s+$//g;
exit if ($user eq '');

use strict;
use warnings;
use lib ('/var/packages/ipkgui/target/include','/var/packages/ipkgui/target/lib');

use CGI;
use CGI::Carp qw(fatalsToBrowser);
use Log::Tiny;
use POSIX qw(strftime tzset);
use File::Copy qw(move);
use File::Basename;
use File::stat;
use Digest::MD5 qw(md5 md5_hex);
use Digest::SHA qw(sha256_hex);
use Encode qw(encode_utf8 decode_utf8);
use JSON::XS;
use Fcntl qw(:flock);
#use Data::Dumper;

our($user);

## initialize vars #################
my $program = "ipkgui";
my $packagename = "ipkgui";
my $version = "0.66";
my $packagehome = "/var/packages/$packagename";
my $homepath = "$packagehome/target";
my $configpath = $packagehome."/etc/";
my $pmconfigfile = $configpath."pmconfig.json";
my $pmtemplatefile = "$packagehome/target/extras/pmtemplates.json";
my $ibconfigfile = $configpath."options.json";
my $ibstatusfile = "";
my $iblistpath = "";
my $ibarchivefile = "";
my $archivepath = "";
my $logfile = "$homepath/log/$program.log";
my $debugfile = "$homepath/log/$program.debug";
my $tmppath = "$homepath/tmp";
my $lockfile = "$tmppath/$program.lock";
my $changesfile = $configpath."lastchange";
my $dsmbuild = `/bin/get_key_value /etc.defaults/VERSION buildnumber`;
chomp($dsmbuild);
my $downcmd = ($dsmbuild >= 4458 ? "/usr/bin/wget --no-check-certificate" : "/usr/syno/bin/wget --no-check-certificate");
my $gzipcmd = "/bin/gzip";

my $cmdret = '';
unless (-d "$homepath/log") {mkdir($homepath."/log",0777) || die "Could not create log dir ($!)\n\n";}
unless (-d $tmppath) {mkdir($tmppath,0777) || die "Could not create tmp dir ($!)\n\n";}
my $log = Log::Tiny->new($logfile) || die "Could not create logfile! (".Log::Tiny->errstr.")\n\n";
my $now = strftime "%Y-%m-%d %H:%M:%S", localtime;
$log->format("$now ipkgui[%p]: %c: %m\n");
chdir $homepath;

my (@parts,$part,$buffer,$value,$name,%param);
my (%match,$packs,$packlog,$opts,$ret,%pkgmvalues,$archive_prio,$archive_feed);
my (@feeds,@prop,$pkg,$ver,$p,$feedname,$feedurl,$fn,$fd,@feedfile_packed);
my $pmoptpre = '';

$ENV{'PATH'} = "/opt/bin:/opt/sbin:".$ENV{'PATH'};

require "functions.pl";

###################################

sub write_data {
     my ($datafile,$cmd,$append) = @_;
     my $append_tag = ">";
     $append_tag = ">>" if ($append);
     if (open(DATA,$append_tag.$datafile)) {
          flock(DATA, LOCK_EX);
          print DATA $cmd;
          close (DATA);
     } else {
          close(DATA);
     }
}
sub exec_cmd {
     my ($pmcmd,$cmdline,$cmd,$curcount,$totcount,$temp,$archive) = @_;
     my $curcmd_data = '';
     my $ret = '';
#$cmdline = 'list | grep "php"|';
     if ($archive eq "1") {
          $cmdret .= "using packages from archive<br>";
          $curcmd_data = '{"current":'.$curcount.',"total":'.$totcount.',"command":"'.$cmd.'","output":"'.$cmdret.'"}';
          write_data($temp,$curcmd_data,0);
     }
     if (open(DAT, $pmcmd." ".$cmdline)) {
          $cmdret .= "#>".$pmcmd." ".substr($cmdline,0,-1)."<br>";
          while (defined($ret = <DAT>)) {
               $ret =~ s/\n/<br>/g;
               $cmdret .= $ret if ($cmdret !~ m/\Q$ret\E/) ;
#               $cmdret .= $ret if ($cmdret =~ m/Successfully terminated/) ;
               $curcmd_data = '{"current":'.$curcount.',"total":'.$totcount.',"command":"'.$cmd.'","output":"'.$cmdret.'"}';
               write_data($temp,$curcmd_data,0);
          }
          close(DAT);
          return 1;
     } else {
          close(DAT);
          return 0;
     }
}
sub get_package_filename {
     my @packs = @{(shift)};
     my @feeds = @{(shift)};

     my ($p,$pkg,$ver,@pfiles,@ifiles,@afiles,@apacks);
     foreach $p (@packs) {
          ($pkg,$ver) = split(/\|\||\|/,$p);
          @prop = ();
          ($feedname,$feedurl,@prop) = get_feed_by_package($iblistpath,$pkg,$ver,@feeds);
          if ($feedname eq "archive") {
               push (@afiles,$prop[0]{'filename'});
               push (@apacks,$p);
          } else {
               push (@pfiles,$feedurl."/".$prop[0]{'filename'}) if ($feedurl && $prop[0]{'filename'});
               push (@ifiles,$prop[0]{'filename'}) if ($prop[0]{'filename'});
          }
     }
     return (\@afiles,\@apacks,\@pfiles,\@ifiles);
}
sub hide_unhide_package {
     my $hide = shift;
     my @packs = @{(shift)};
     my @feeds = @{(shift)};

     my ($ex,$pkg,$ver);
     foreach my $fd (@feeds) {
          if ($fd->{'feedactive'}) {
               foreach my $pk (@packs) {
                    if (index($pk,'||') != -1) {
                         ($pkg,$ver) = split(/\|\||\|/,$pk);
                         if ($hide) {
                              # workaround part 1
                              # disable the package name in the list because ipkg/opkg wants to download the latest version in spite of a specified local file
                              # if list of packages a gz compressed file, rename the original file...
                              if (test_gzipped($iblistpath."/".$fd->{'feedname'})) {
                                   if (!move($iblistpath."/".$fd->{'feedname'}, $iblistpath."/".$fd->{'feedname'}."_packed")) {
                                        print '{"success":false}';
                                        exit;
                                   }
                                   # and decompress it, so that the name like the original one
                                   my $gz = `/bin/zcat $iblistpath/$fd->{'feedname'}_packed > $iblistpath/$fd->{'feedname'}`;
                                   if ($? == 0) {
                                        push(@feedfile_packed,$fd->{'feedname'});
                                   }
                              }
                              # s/^Package:.$pkg\n(Version:.$ver)\n/#Package: $pkg\n$1\n/ $iblistpath/$fd->{'feedname'}`;
                              $ex = `/bin/sed -i 's/^Package.*:.*$pkg/#Package: $pkg/g' $iblistpath/$fd->{'feedname'}`;
                         } else {
                              # workaround part 2
                              # revoke previously performed change in the package list
                              $ex = `/bin/sed -i 's/^#Package.*:.*$pkg/Package: $pkg/g' $iblistpath/$fd->{'feedname'}`;
                         }
                    }
               }
          }
     }
}
sub download_version {
     my ($archive,$cmd,$curcount,$totcount,$temp,@packs) = @_;

     my $curcmd_data = '';
     my $ret = '';
     my $destpath = ($archive eq "1") ? $archivepath : $tmppath;
     my $desttext = ($archive eq "1") ? ' to archive dir...' : '...';
     my ($p,$fname,$cmdline,$result,$err);

     foreach $p (@packs) {
          $fname = fileparse($p);
          $cmdline = "-nd -U mozilla -O ".$destpath."/".$fname." ".$p;
          if (-f $destpath."/".$fname) {
               $cmdret .= "#>$fname already exists in archive dir, skipping download<br>";
               $curcmd_data = '{"current":'.$curcount.',"total":'.$totcount.',"command":"'.$cmd.'","output":"'.$cmdret.'"}';
               write_data($temp,$curcmd_data,0);
               $err = 0;
          } else {
               $cmdret .= "#>downloading $fname$desttext";
               $curcmd_data = '{"current":'.$curcount.',"total":'.$totcount.',"command":"'.$cmd.'","output":"'.$cmdret.'"}';
               write_data($temp,$curcmd_data,0);
               $result = `$downcmd $cmdline`;
               $err += $?;
               $result = ($err) ? "an error $? has occurred!" : "done";
               $cmdret .= "$result.<br>";
               $curcmd_data = '{"current":'.$curcount.',"total":'.$totcount.',"command":"'.$cmd.'","output":"'.$cmdret.'"}';
               write_data($temp,$curcmd_data,0);
          }
     }
     return ($err > 0) ? 0 : 1;
}
sub remove_from_archive {
     my $packs = shift;

     my ($p,$pkg,$ver);
     my @packs = split(" ",$packs);
     my $err = 0;
     chdir $archivepath;

     # if archive available
     if (-e $ibarchivefile) {
          foreach $p (@packs) {
               ($pkg,$ver) = split(/\|/,$p);
               @prop = ();
               ($feedname,$feedurl,@prop) = get_feed_by_package($iblistpath,$pkg,$ver,@feeds);
               $err++ if (!unlink($prop[0]{'filename'}));
          }
          rebuild_archive();
     }
     return $err;
}
sub rebuild_archive {
     my ($fname,$ret,$tarcmd,$result);
     my $err = 0;
     my @files = glob($archivepath."/*.ipk ".$archivepath."/*.opk");
     if (scalar(@files) == 0 && -e $ibarchivefile) {
          $err++ if (!unlink($ibarchivefile));
          $err++ if (!unlink($archivepath."/Packages.gz"));
     } else {
          foreach my $file (@files) {
               $tarcmd = "/bin/tar -xzOf $file ./control.tar.gz | /bin/tar xzOf - ./control";
               $ret .= `$tarcmd`;
               $ret .= "Filename: ".fileparse($file)."\n";
               $ret .= "Size: ".(-s $file)."\n";
               $ret .= "MD5Sum: ".md5hex_file($file)."\n";
               $ret .= "SHA256sum: ".sha256hex_file($file)."\n";
               $ret .= "\n";
          }
          write_data($ibarchivefile,$ret,0);
          chdir $archivepath;
          if (-e $ibarchivefile) {
               $result = `$gzipcmd -c $ibarchivefile > $archivepath/Packages.gz` if ($archive_feed eq "1");
               $err += $?;
          }
     }
     return $err;
}
sub set_external {
     my $packs = shift;
     my $ibstatusfile = shift;
     my @feeds = @{(shift)};

     my ($newstatus,@newstat,$i,$p,$pkg,$ver);
     my @packs = split(" ",$packs);

     foreach $p (@packs) {
          ($pkg,$ver) = split(/\|\||\|/,$p);
          @prop = ();
          ($feedname,$feedurl,@prop) = get_feed_by_package($iblistpath,$pkg,$ver,@feeds);
          push(@newstat,@prop);
     }
     if (scalar(@newstat)) {
          $newstatus = '';
          for ($i = 0; $i <= $#newstat; $i++) {
               $newstatus .= "Package: ".$newstat[$i]->{'package'}."\n";
               $newstatus .= "Version: n/a\n";
               $newstatus .= "Depends: ".$newstat[$i]->{'depends'}."\n" if defined $newstat[$i]->{'depends'};
               $newstatus .= "Suggests: ".$newstat[$i]->{'suggests'}."\n" if defined $newstat[$i]->{'suggests'};
               $newstatus .= "Conflicts: ".$newstat[$i]->{'conflicts'}."\n" if defined $newstat[$i]->{'conflicts'};
               $newstatus .= "Status: unknown hold installed\n";
               $newstatus .= "Architecture: ".$newstat[$i]->{'architecture'}."\n";
               $newstatus .= "Installed-Time: ".$newstat[$i]->{'installed-time'}."\n" if defined $newstat[$i]->{'installed-time'};
               $newstatus .= "\n";
          }
          write_data($ibstatusfile,$newstatus,1);
     }
}
sub unset_external {
     my $packs = shift;
     my $pmstatcmd = shift;
     my $ibstatusfile = shift;
     my @feeds = @{(shift)};

     my ($newstatus,@newstat,$in,$iv,$i,$f);
     my $empty = 0;
     my @packstat = ();
     my @packs = split(" ",$packs);

     # generate list of installed packages and create a array
     @packstat = read_list($pmstatcmd);

     # remove selected packages from status file
     my %packs = map {$_ => 1} @packs;
     foreach $f (@packstat) {
          if (!exists($packs{$f->{'package'}})) {
               push @newstat, $f;
          }
     }
     if (@newstat) {
          $newstatus = '';
          for ($i = 0; $i <= $#newstat; $i++) {
               $newstatus .= "Package: ".$newstat[$i]->{'package'}."\n";
               $newstatus .= "Version: ".$newstat[$i]->{'version'}."\n";
               $newstatus .= "Depends: ".$newstat[$i]->{'depends'}."\n" if defined $newstat[$i]->{'depends'};
               $newstatus .= "Suggests: ".$newstat[$i]->{'suggests'}."\n" if defined $newstat[$i]->{'suggests'};
               $newstatus .= "Conflicts: ".$newstat[$i]->{'conflicts'}."\n" if defined $newstat[$i]->{'conflicts'};
               $newstatus .= "Status: ".$newstat[$i]->{'status'}."\n";
               $newstatus .= "Architecture: ".$newstat[$i]->{'architecture'}."\n";
               $newstatus .= "Installed-Time: ".$newstat[$i]->{'installed-time'}."\n" if defined $newstat[$i]->{'installed-time'};
               $newstatus .= "\n";
          }
          write_data($ibstatusfile.".tmp",$newstatus,0);
          if (-e $ibstatusfile.".tmp") {
               move($ibstatusfile.".tmp", $ibstatusfile);
               return 0;
          } else {
               return 1;
          }
     }
}
sub get_options {
     my $pm = shift;
     my @options = @{$_[0]};
     my $option = '';
     foreach my $o (@options) {
          if ($pm eq "opkg") {
               $option .= ' -'.$o;
          } else {
               $option .= ' '.$o;
          }
     }
     return $option;
}

print "Content-type: application/json; charset=UTF-8\n\n";

if (-e $lockfile) {
     my $lockuser = `cat $lockfile >&1`;
     print '({"success":false,"msg":"StatusExecuteLockedBy","user":"'.$lockuser.'"})';
     exit;
}

if ($ENV{'REQUEST_METHOD'} eq "POST") {
     read(STDIN,$buffer,$ENV{"CONTENT_LENGTH"});
     @parts = split(/\&/,$buffer);
     foreach $part (@parts) {
          ($name,$value) = split(/=/,$part);
          $value =~ tr/+/ /;
          $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
          # prevent include of server side includes
          $value =~ s/<!--(.|\n)*-->//g;
          $param{$name} = $value;
     }
     my $ib_temp = $tmppath."/".$param{'tid'} if ($param{'tid'} ne "");
     my $pmconfig = detect_pkm($pmconfigfile,$pmtemplatefile);
     if (scalar(keys(%{$pmconfig})) == 0) {
          print '({"success":false,"msg":"StatusNoIPKGText"})';
          exit;
     }
     my $pmstatcmd = "$pmconfig->{'pmexec'} status 2> /dev/null|";
     my @config_data = read_config($ibconfigfile);
     if (exists($config_data[0]{'options'}[0])) {
          $archive_prio = $config_data[0]{'options'}[0]{'archive_prio'} if (defined $config_data[0]{'options'}[0]{'archive_prio'});
          $archive_feed = $config_data[0]{'options'}[0]{'archive_feed'} if (defined $config_data[0]{'options'}[0]{'archive_feed'});
     }
     if ($pmconfig->{'pmexec'} =~ m/opkg/i) {
          $pmoptpre = '-';
          $param{'force_defaults'} = 0;
     }
     $ibstatusfile = $pmconfig->{'statusfile'};
     $iblistpath = $pmconfig->{'listpath'};
     $ibarchivefile = $iblistpath."/archive";
     $archivepath = $pmconfig->{'archivepath'};
     unless (-d $archivepath) {mkdir($archivepath,0777) || die "Could not create archive dir ($!)\n\n";}

     @feeds = {'feedname' => 'archive','feedactive' => 1,'feedfile' => '','feedtype' => 'src','feedurl' => 'archive'} if ($archive_prio eq "1");
     push @feeds, get_feeds($pmconfig->{'feeds'});
     push @feeds,{'feedname' => 'archive','feedactive' => 1,'feedfile' => '','feedtype' => 'src','feedurl' => 'archive'} if ($archive_prio eq "0");

     if ($param{'action'} eq "hold" && $param{'value'} ne "") {
          %pkgmvalues = %{JSON::XS->new->utf8->decode($param{'value'})};
          $cmdret = `$pmconfig->{'pmexec'} flag hold $pkgmvalues{'value'} >&1`;
          if ($? != 0) {
               print '{"success":false}';
          }
          $log->INFO("set flag hold for $pkgmvalues{'value'}");
     } elsif ($param{'action'} eq "unhold" && $param{'value'} ne "") {
          %pkgmvalues = %{JSON::XS->new->utf8->decode($param{'value'})};
          $cmdret = `$pmconfig->{'pmexec'} flag user $pkgmvalues{'value'} >&1`;
          if ($? != 0) {
               print '{"success":false}';
          }
          $log->INFO("unset flag hold for $pkgmvalues{'value'}");
     } elsif ($param{'action'} eq "set_external" && $param{'value'} ne "") {
          %pkgmvalues = %{JSON::XS->new->utf8->decode($param{'value'})};
          $cmdret = set_external($pkgmvalues{'value'},$ibstatusfile,\@feeds);
          if ($? != 0) {
               print '{"success":false}';
          }
          $pkgmvalues{'value'} =~ s/\|/ /g;
          $log->INFO("set external installation for $pkgmvalues{'value'}");
     } elsif ($param{'action'} eq "unset_external" && $param{'value'} ne "") {
          %pkgmvalues = %{JSON::XS->new->utf8->decode($param{'value'})};
          $cmdret = unset_external($pkgmvalues{'value'},$pmstatcmd,$ibstatusfile,\@feeds);
          if ($? != 0) {
               print '{"success":false}';
          }
          $log->INFO("unset external installation for $pkgmvalues{'value'}");
     } elsif ($param{'action'} eq "rem_archive" && $param{'value'} ne "") {
          %pkgmvalues = %{JSON::XS->new->utf8->decode($param{'value'})};
          $cmdret = remove_from_archive($pkgmvalues{'value'});
          if ($? != 0) {
               print '{"success":false}';
          }
          $log->INFO("unset external installation for $pkgmvalues{'value'}");
     } elsif ($param{'action'} eq "rebuild_archive" && $param{'value'} ne "") {
          $cmdret = rebuild_archive();
          if ($? != 0) {
               print '{"success":false}';
          }
          $log->INFO("rebuild archive");
     } elsif ($param{'action'} eq "apply" && $param{'value'} ne "" && $param{'tid'} ne "") {
          %pkgmvalues = %{JSON::XS->new->utf8->decode($param{'value'})};

          # count only grouped actions
          my $ib_totcount = 0;
          for (keys %pkgmvalues) {
               if ($#{$pkgmvalues{$_}->{'NF'}->{'packages'}} + 1) {$ib_totcount++};
               if ($#{$pkgmvalues{$_}->{'FO'}->{'packages'}} + 1) {$ib_totcount++};
               if ($#{$pkgmvalues{$_}->{'FN'}->{'packages'}} + 1) {$ib_totcount++};
               if ($#{$pkgmvalues{$_}->{'FON'}->{'packages'}} + 1) {$ib_totcount++};
               if ($#{$pkgmvalues{$_}->{'FD'}->{'packages'}} + 1) {$ib_totcount++};
               if ($#{$pkgmvalues{$_}->{'FR'}->{'packages'}} + 1) {$ib_totcount++};
          }
          if (index($pkgmvalues{$_}->{'NF'}->{'packages'},"||") != -1) {$ib_totcount++};
          if (index($pkgmvalues{$_}->{'FO'}->{'packages'},"||") != -1) {$ib_totcount++};
          if (index($pkgmvalues{$_}->{'FN'}->{'packages'},"||") != -1) {$ib_totcount++};
          if (index($pkgmvalues{$_}->{'FON'}->{'packages'},"||") != -1) {$ib_totcount++};
#print $ib_totcount;
          if ($ib_totcount) {
               # increased the total number of steps by 1 to do the later synthetic loop
               $ib_totcount++;
               # write lockfile to prevent parallel execution
               write_data($lockfile,$user,0);
               my $ib_curcount = 0;
               my $curcmd_data = '';
               # preparing actions...please wait
               $curcmd_data = '{"current":'.$ib_curcount.',"total":'.$ib_totcount.',"command":"preparing","output":"preparing actions...please wait"}';
               write_data($ib_temp,$curcmd_data,0);

               my @pkgtemp = ();
               my $ex = 0;
               my $dn = 0;
               my $mi = 0;
               my $pm = $match{"pm"};
               my $printcmd = '';
               my $instcmd = 'install';
               my $upgcmd = 'upgrade';
               $instcmd .= " $pmoptpre-force-defaults" if ($param{'force_defaults'} eq "1");
#               $instcmd .= " $pmoptpre-force-maintainer" if ($param{'force_defaults'} eq "1");
               $upgcmd .= " $pmoptpre-force-defaults" if ($param{'force_defaults'} eq "1");
#               $upgcmd .= " $pmoptpre-force-maintainer" if ($param{'force_defaults'} eq "1");
               my $sourcepath = ($param{'archive'} eq "1") ? $archivepath : $tmppath;
               if ($pkgmvalues{'remove'}) {
                    foreach my $p (keys %{$pkgmvalues{'remove'}}) {
                         if ($#{$pkgmvalues{'remove'}->{$p}->{'packages'}} + 1) {
                              foreach $p (@{$pkgmvalues{'remove'}->{$p}->{'packages'}}) {
                                   ($pkg,$ver) = split(/\|\||\|/,$p);
                                   push(@pkgtemp,$pkg);
                              }
                              $packs = join(" ",@pkgtemp);
                              $opts = get_options($pm,\@{$pkgmvalues{'remove'}->{$p}->{'option'}});
                              $ib_curcount++;
#                              print "remove$opts $packs\n";
                              $ex = exec_cmd($pmconfig->{'pmexec'},"remove$opts $packs|","remove",$ib_curcount,$ib_totcount,$ib_temp,"0");
                              $packlog = join(" ",@{$pkgmvalues{'remove'}->{$p}->{'packages'}});
                              $packlog =~ s/\|\||\|/ /g;
                              $log->INFO("remove$opts $packlog");
                         }
                    }
               }
               if ($pkgmvalues{'downgrade'}) {
                    foreach my $p (keys %{$pkgmvalues{'downgrade'}}) {
                         if ($#{$pkgmvalues{'downgrade'}->{$p}->{'packages'}} + 1) {
                              $opts = get_options($pm,\@{$pkgmvalues{'downgrade'}->{$p}->{'option'}});
                              $printcmd = 'downgrade';
                              $ib_curcount++;
#                              print "$printcmd$opts $packs\n";
                              my ($archfiles,$archpacks,$packfiles,$instfiles) = get_package_filename(\@{$pkgmvalues{'downgrade'}->{$p}->{'packages'}},\@feeds);
                              $dn = download_version($param{'archive'},"downgrade",$ib_curcount,$ib_totcount,$ib_temp,@$packfiles) if (scalar(@$instfiles));
                              if (scalar(@$instfiles) == 0 && scalar(@$archfiles) == 0) {
                                   $cmdret .= "#>no files found, perhaps repository not active or not available, skipping downgrade<br>done";
                                   $curcmd_data = '{"current":'.$ib_curcount.',"total":'.$ib_totcount.',"command":"downgrade","output":"'.$cmdret.'"}';
                                   write_data($ib_temp,$curcmd_data,0);
                              } elsif (($dn && $param{'download'} eq "0") || $param{'download'} eq "0") {
                                   foreach $p (@{$pkgmvalues{'downgrade'}->{$p}->{'packages'}}) {
                                        ($pkg,$ver) = split(/\|\||\|/,$p);
                                        push(@pkgtemp,$pkg);
                                   }
                                   $packs = join(" ",@pkgtemp);
                                   $ex = exec_cmd($pmconfig->{'pmexec'},"remove $pmoptpre-force-depends $packs|","downgrade",$ib_curcount,$ib_totcount,$ib_temp,"0");
                                   hide_unhide_package(1,\@{$pkgmvalues{'downgrade'}->{$p}->{'packages'}},\@feeds);
                                   if (scalar(@$archfiles)) {
                                        chdir $archivepath;
                                        $packs = join(" ",@$archfiles);
                                        $ex = exec_cmd($pmconfig->{'pmexec'},"$instcmd$opts $packs|","downgrade",$ib_curcount,$ib_totcount,$ib_temp,"1");
                                   }
                                   if (scalar(@$instfiles)) {
                                        chdir $sourcepath;
                                        $packs = join(" ",@$instfiles);
                                        $ex = exec_cmd($pmconfig->{'pmexec'},"$instcmd$opts $packs|","downgrade",$ib_curcount,$ib_totcount,$ib_temp,"0");
                                   }
                                   hide_unhide_package(0,\@{$pkgmvalues{'downgrade'}->{$p}->{'packages'}},\@feeds);
                              } elsif ($param{'download'} eq "1") {
                                   my ($p,$fname);
                                   foreach $p (@$archfiles) {
                                        $fname = fileparse($p);
                                        $cmdret .= "#>$fname already exists in archive dir, skipping download<br>";
                                        $curcmd_data = '{"current":'.$ib_curcount.',"total":'.$ib_totcount.',"command":"downgrade","output":"'.$cmdret.'"}';
                                        write_data($ib_temp,$curcmd_data,0);
                                   }
                              }
                              rebuild_archive() if ($param{'archive'} eq "1");
                              $packlog = join(" ",@{$pkgmvalues{'downgrade'}->{$p}->{'packages'}});
                              $packlog =~ s/\|\||\|/ /g;
                              $log->INFO("$printcmd$opts ".(scalar(@$archfiles)?'from archive':'')." $packlog");
                         }
                    }
               }
               if ($pkgmvalues{'upgrade'}) {
                    foreach my $p (keys %{$pkgmvalues{'upgrade'}}) {
                         if ($#{$pkgmvalues{'upgrade'}->{$p}->{'packages'}} + 1) {
                              $opts = get_options($pm,\@{$pkgmvalues{'upgrade'}->{$p}->{'option'}});
                              $printcmd = 'upgrade';
                              $ib_curcount++;
#                              print "$printcmd$opts $packs\n";
                              if ($param{'archive'} eq "1") {
                                   my ($archfiles,$archpacks,$packfiles,$instfiles) = get_package_filename(\@{$pkgmvalues{'upgrade'}->{$p}->{'packages'}},\@feeds);
                                   $dn = download_version($param{'archive'},"upgrade",$ib_curcount,$ib_totcount,$ib_temp,@$packfiles) if (scalar(@$instfiles));
                                   if (scalar(@$instfiles) == 0 && scalar(@$archfiles) == 0) {
                                        $cmdret .= "#>no files found, perhaps repository not active or not available, skipping upgrade<br>done";
                                        $curcmd_data = '{"current":'.$ib_curcount.',"total":'.$ib_totcount.',"command":"upgrade","output":"'.$cmdret.'"}';
                                        write_data($ib_temp,$curcmd_data,0);
                                   } elsif (($dn && $param{'download'} eq "0") || $param{'download'} eq "0") {
                                        if (scalar(@$archfiles)) {
                                             chdir $archivepath;
                                        } else {
                                             chdir $sourcepath;
                                        }
                                        $packs = join(" ",@$instfiles)." ".join(" ",@$archfiles);
                                        $ex = exec_cmd($pmconfig->{'pmexec'},"$instcmd $opts $packs|","upgrade",$ib_curcount,$ib_totcount,$ib_temp,"1");
                                   } elsif ($param{'download'} eq "1") {
                                        my ($p,$fname);
                                        foreach $p (@$archfiles) {
                                             $fname = fileparse($p);
                                             $cmdret .= "#>$fname already exists in archive dir, skipping download<br>";
                                             $curcmd_data = '{"current":'.$ib_curcount.',"total":'.$ib_totcount.',"command":"upgrade","output":"'.$cmdret.'"}';
                                             write_data($ib_temp,$curcmd_data,0);
                                        }
                                   }
                                   rebuild_archive();
                                   $packlog = join(" ",@{$pkgmvalues{'upgrade'}->{$p}->{'packages'}});
                                   $packlog =~ s/\|\||\|/ /g;
                                   $log->INFO("$printcmd$opts ".(scalar(@$archfiles)?'from archive':'')." $packlog");
                              } elsif ($param{'archive'} ne "1") {
                                   my ($archfiles,$archpacks,$packfiles,$instfiles) = get_package_filename(\@{$pkgmvalues{'upgrade'}->{$p}->{'packages'}},\@feeds);
                                   $packlog = join(" ",@{$pkgmvalues{'upgrade'}->{$p}->{'packages'}});
                                   $packlog =~ s/\|\||\|/ /g;
                                   if (scalar(@$archfiles)) {
                                        chdir $archivepath;
                                        $packs = join(" ",@$archfiles);
                                        $ex = exec_cmd($pmconfig->{'pmexec'},"$instcmd$opts $packs|","upgrade",$ib_curcount,$ib_totcount,$ib_temp,"1");
                                   }
                                   my @upgrade = array_diff(\@{$pkgmvalues{'upgrade'}->{$p}->{'packages'}},$archpacks);
                                   if (scalar(@upgrade)) {
                                        s/\|.*$// for @upgrade;
                                        $packs = join(" ",@upgrade);
                                        $ex = exec_cmd($pmconfig->{'pmexec'},"$upgcmd$opts $packs|","upgrade",$ib_curcount,$ib_totcount,$ib_temp,"0");
                                   }
                                   $log->INFO("$printcmd$opts $packlog");
                              }
                         }
                    }
               }
               if ($pkgmvalues{'reinstall'}) {
                    foreach my $p (keys %{$pkgmvalues{'reinstall'}}) {
                         if ($#{$pkgmvalues{'reinstall'}->{$p}->{'packages'}} + 1) {
                              $opts = get_options($pm,\@{$pkgmvalues{'reinstall'}->{$p}->{'option'}});
                              $printcmd = 'reinstall';
                              $ib_curcount++;
#                              print "$printcmd$opts $packs\n";
                              if ($param{'archive'} eq "1") {
                                   my ($archfiles,$archpacks,$packfiles,$instfiles) = get_package_filename(\@{$pkgmvalues{'reinstall'}->{$p}->{'packages'}},\@feeds);
                                   $dn = download_version($param{'archive'},"reinstall",$ib_curcount,$ib_totcount,$ib_temp,@$packfiles) if (scalar(@$instfiles));
                                   if (scalar(@$instfiles) == 0 && scalar(@$archfiles) == 0) {
                                        $cmdret .= "#>no files found, perhaps repository not active or not available, skipping reinstall<br>done";
                                        $curcmd_data = '{"current":'.$ib_curcount.',"total":'.$ib_totcount.',"command":"reinstall","output":"'.$cmdret.'"}';
                                        write_data($ib_temp,$curcmd_data,0);
                                   } elsif (($dn && $param{'download'} eq "0") || $param{'download'} eq "0") {
                                        if (scalar(@$archfiles)) {
                                             chdir $archivepath;
                                        } else {
                                             chdir $sourcepath;
                                        }
                                        $packs = join(" ",@$instfiles)." ".join(" ",@$archfiles);
                                        $ex = exec_cmd($pmconfig->{'pmexec'},"$instcmd $opts $packs|","reinstall",$ib_curcount,$ib_totcount,$ib_temp,"1");
                                   } elsif ($param{'download'} eq "1") {
                                        my ($p,$fname);
                                        foreach $p (@$archfiles) {
                                             $fname = fileparse($p);
                                             $cmdret .= "#>$fname already exists in archive dir, skipping download<br>";
                                             $curcmd_data = '{"current":'.$ib_curcount.',"total":'.$ib_totcount.',"command":"reinstall","output":"'.$cmdret.'"}';
                                             write_data($ib_temp,$curcmd_data,0);
                                        }
                                   }
                                   rebuild_archive();
                                   $packlog = join(" ",@{$pkgmvalues{'reinstall'}->{$p}->{'packages'}});
                                   $packlog =~ s/\|\||\|/ /g;
                                   $log->INFO("$printcmd$opts ".(scalar(@$archfiles)?'from archive':'')." $packlog");
                              } elsif ($param{'archive'} ne "1") {
                                   my ($archfiles,$archpacks,$packfiles,$instfiles) = get_package_filename(\@{$pkgmvalues{'reinstall'}->{$p}->{'packages'}},\@feeds);
                                   $packlog = join(" ",@{$pkgmvalues{'reinstall'}->{$p}->{'packages'}});
                                   $packlog =~ s/\|\||\|/ /g;
                                   if (scalar(@$archfiles)) {
                                        chdir $archivepath;
                                        $packs = join(" ",@$archfiles);
                                        $ex = exec_cmd($pmconfig->{'pmexec'},"$instcmd $opts $packs|","reinstall",$ib_curcount,$ib_totcount,$ib_temp,"1");
                                   }
                                   my @reinstall = array_diff(\@{$pkgmvalues{'reinstall'}->{$p}->{'packages'}},$archpacks);
                                   if (scalar(@reinstall)) {
                                        s/\|.*$// for @reinstall;
                                        $packs = join(" ",@reinstall);
                                        $ex = exec_cmd($pmconfig->{'pmexec'},"$instcmd $pmoptpre-force-reinstall$opts $packs|","reinstall",$ib_curcount,$ib_totcount,$ib_temp,"0");
                                   }
                                   $log->INFO("$printcmd$opts $packlog");
                              }
                         }
                    }
               }
               if ($pkgmvalues{'install'}) {
                    foreach my $p (keys %{$pkgmvalues{'install'}}) {
                         if ($#{$pkgmvalues{'install'}->{$p}->{'packages'}} + 1) {
                              $opts = get_options($pm,\@{$pkgmvalues{'install'}->{$p}->{'option'}});
                              $printcmd = 'install';
                              $ib_curcount++;
#                              print "$printcmd$opts $packs\n";
                              my @installm = ();
                              if ($param{'archive'} eq "1") {
                                   @installm = @{$pkgmvalues{'install'}->{$p}->{'packages'}};
                              } else {
                                   @installm = grep{index($_,'||') != -1} @{$pkgmvalues{'install'}->{$p}->{'packages'}};
                              }
                              if ($param{'archive'} eq "1" || scalar(@installm)) {
                                   my ($archfiles,$archpacks,$packfiles,$instfiles) = get_package_filename(\@installm,\@feeds);
                                   $dn = download_version($param{'archive'},"install",$ib_curcount,$ib_totcount,$ib_temp,@$packfiles) if (scalar(@$instfiles));
                                   if (scalar(@$instfiles) == 0 && scalar(@$archfiles) == 0) {
                                        $cmdret .= "#>no files found, perhaps repository not active or not available, skipping install<br>done";
                                        $curcmd_data = '{"current":'.$ib_curcount.',"total":'.$ib_totcount.',"command":"install","output":"'.$cmdret.'"}';
                                        write_data($ib_temp,$curcmd_data,0);
                                   } elsif (($dn && $param{'download'} eq "0") || $param{'download'} eq "0") {
                                        hide_unhide_package(1,\@installm,\@feeds);
                                        if (scalar(@$archfiles)) {
                                             chdir $archivepath;
                                        } else {
                                             chdir $sourcepath;
                                        }
                                        $packs = join(" ",@$instfiles)." ".join(" ",@$archfiles);
                                        $ex = exec_cmd($pmconfig->{'pmexec'},"$instcmd$opts $packs|","install",$ib_curcount,$ib_totcount,$ib_temp,(scalar(@$archfiles)));
                                        hide_unhide_package(0,\@installm,\@feeds);
                                   } elsif ($param{'download'} eq "1") {
                                        my ($p,$fname);
                                        foreach $p (@$archfiles) {
                                             $fname = fileparse($p);
                                             $cmdret .= "#>$fname already exists in archive dir, skipping download<br>";
                                             $curcmd_data = '{"current":'.$ib_curcount.',"total":'.$ib_totcount.',"command":"install","output":"'.$cmdret.'"}';
                                             write_data($ib_temp,$curcmd_data,0);
                                        }
                                   }
                                   rebuild_archive();
                                   $packlog = join(" ",@installm);
                                   $packlog =~ s/\|\||\|/ /g;
                                   $log->INFO("$printcmd$opts ".(scalar(@$archfiles)?'from archive':'')." $packlog");
                              }
                              my @install = grep{index($_,'|') != -1 && index($_,'||') == -1} @{$pkgmvalues{'install'}->{$p}->{'packages'}};
                              if ($param{'archive'} ne "1" && scalar(@install)) {
                                   my ($archfiles,$archpacks,$packfiles,$instfiles) = get_package_filename(\@install,\@feeds);
                                   $packlog = join(" ",@install);
                                   $packlog =~ s/\|\||\|/ /g;
                                   if (scalar(@$archfiles)) {
                                        chdir $archivepath;
                                        $packs = join(" ",@$archfiles);
                                        $ex = exec_cmd($pmconfig->{'pmexec'},"$instcmd$opts $packs|","install",$ib_curcount,$ib_totcount,$ib_temp,"1");
                                   }
                                   @install = array_diff(\@install,$archpacks);
                                   if (scalar(@install)) {
                                        s/\|.*$// for @install;
                                        $packs = join(" ",@install);
                                        $ex = exec_cmd($pmconfig->{'pmexec'},"$instcmd$opts $packs|","install",$ib_curcount,$ib_totcount,$ib_temp,"0");
                                   }
                                   $log->INFO("$printcmd$opts $packlog");
                              }
                         }
                    }
               }
               # delete the decompressed packages file and rename the original back
               foreach my $pf (@feedfile_packed) {
                    unlink ($iblistpath."/".$pf) if (-e $iblistpath."/".$pf);
                    if (!move($iblistpath."/".$pf."_packed", $iblistpath."/".$pf)) {
                         print '{"success":false}';
                         exit;
                    }
               }
               # synthetic loop to show the last segment of the progress bar
               $ib_curcount++;
               $curcmd_data = '{"current":'.$ib_curcount.',"total":'.$ib_totcount.',"command":"finish","output":"'.$cmdret.'"}';
               write_data($ib_temp,$curcmd_data,0);
               select(undef, undef, undef, 2);
          } else {
               print '{"success":false}';
          }
     }
}
# clear tmp dir
if (-d '/var/packages/ipkgui/target/tmp/') {
     unlink glob('/var/packages/ipkgui/target/tmp/*');
}
# write timestamp of last changes in status file
my $lastchange = (-e $ibstatusfile ? stat($ibstatusfile)->mtime : 0);
write_data($changesfile,$lastchange,0);
