#!/usr/bin/perl
#*******************************************************************#
#  iPKGui - 3rdparty ipkg/opkg browser/webUI                        #
#  Modul:       export.cgi                                          #
#  Description: a perl script to save a value and export            #
#               as file                                             #
#  Author:      QTip from the german Synology support forum         #
#  Copyright:   2014-2015 by QTip                                   #
#  License:     GNU GPLv3 (see LICENSE)                             #
#  ---------------------------------------------------------------  #
#  Version:     0.15 - 24/07/2015                                   #
#  for more information check the changelog                         #
#*******************************************************************#

$user = `/usr/syno/synoman/webman/modules/authenticate.cgi`;
$user =~ s/^\s+|\s+$//g;
exit if ($user eq '');

use strict;
use warnings;
use lib ('/var/packages/ipkgui/target/include','/var/packages/ipkgui/target/lib');

use CGI;
use CGI::Carp qw(fatalsToBrowser);
use Log::Tiny;
use POSIX qw(strftime tzset);

my $program = "ipkgui";
my $homepath = "/var/packages/$program/target";
my $exportpath = "$homepath/tmp";
my $logfile = "$homepath/log/$program.log";
unless (-d $exportpath) {mkdir($exportpath,0777) || die "Could not create export dir ($!)\n\n";}
unless (-d "$homepath/log") {mkdir($homepath."/log",0777) || die "Could not create log dir ($!)\n\n";}
my $log = Log::Tiny->new($logfile) || die "Could not create logfile! (".Log::Tiny->errstr.")\n\n";
my $tsf = strftime "%Y-%m-%d %H:%M:%S", localtime;
$log->format("$tsf ipkgui[%p]: %c: %m\n");

my (@filebuffer,$result,$download_file,$dl_file);

if ($ENV{'REQUEST_METHOD'} eq "POST") {
     my $cgi = CGI->new();
     my %params = $cgi->Vars();
     if ($params{'action'} eq "export" && $params{'value'} ne "") {
          print $cgi->header(-type => 'text/html');
          my $now = strftime "%Y%m%d%H%M", localtime;
          $dl_file = "ipkgui_packages_".$now.".json";
          $download_file = $exportpath."/".$dl_file;
          $params{'value'} =~ tr/+/ /;
          $params{'value'} =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
          # prevent include of server side includes
          $params{'value'} =~ s/<!--(.|\n)*-->//g;
          open(DATA,">".$download_file);
          print DATA $params{'value'};
          close (DATA);
          if ($? == 0) {
               print '{"success":true,"file":"'.$dl_file.'"}';
               $log->INFO("$dl_file successfully exported!");
          } else {
               print '{"success":false,"msg":"IPKG.Browser.Texts[\'StatusExportFailedText\']"}';
               $log->ERROR("There was an error during export of the file '$dl_file'!");
          }
     }
} else {
     my $cgi = CGI->new();
     my %params = $cgi->Vars();
     if ($params{'action'} eq "download" && $params{'file'} ne "") {
          $download_file = $exportpath."/".$params{'file'};
          exit unless (-e $download_file);
          my $filesize = -s $download_file;
          open(DLFILE, "<$download_file") || Error('open', 'file');
          @filebuffer = <DLFILE>;
          close (DLFILE) || Error ('close', 'file');

          print "Cache-Control: maxage=120\n";
          print "Expires: 0";
          print "Pragma: public\n";
          print "Content-type:application/force-download\n";
          print "Content-Transfer-Encoding: Binary\n";
          print "Content-Type:application/octet-stream\n";
          print "Content-Disposition: attachment; filename=\"$params{'file'}\"\n";
          print "Content-Length: $filesize\n\n";
          print @filebuffer;
     }
}
exit;