#!/usr/bin/perl
#*******************************************************************#
#  iPKGui - 3rdparty ipkg/opkg browser/webUI                        #
#  Modul:       functions.pl                                        #
#  Description: misc perl functions                                 #
#  Author:      QTip from the german Synology support forum         #
#  Copyright:   2012-2015 by QTip                                   #
#  License:     GNU GPLv3 (see LICENSE)                             #
#  ---------------------------------------------------------------  #
#  Version:     0.55 - 16/08/2015                                   #
#  for more information check the changelog                         #
#*******************************************************************#

sub trim {
     @_ = $_ if not @_ and defined wantarray;
     @_ = @_ if defined wantarray;
     for (@_ ? @_ : $_) { s/^\s+//, s/\s+$// }
     return wantarray ? @_ : $_[0] if defined wantarray;
}
sub in_array {
     my ($needle,@haystack) = @_;
     return grep{$_ eq $needle} @haystack;
}
sub is_array {
     my $ref = shift;
     return 0 unless ref $ref;
     if (ref($ref) eq "ARRAY") {
          return 1;
     } else {
          return 0;
     }
}
sub array_diff {
     my @a = @{(shift)};
     my @b = @{(shift)};
     my %in_bl = map {$_ => 1} @b;
     my @rest = grep {not $in_bl{$_}} @a;
     return @rest;
}
sub array_unique {
     my @a = @{(shift)};
     my @unique = do {
          my %seen;
          grep{!$seen{$_}++} @data;
     };
     return @unique;
}
sub test_gzipped {
     my $file = shift;
     `/bin/gunzip -t "$file" 2> /dev/null`;
     return $? > 0 ? 0 : 1;
}
sub detect_pkm {
     # return the following keys for each sub hash:
     # $pmconf->{'pmroot'}
     # $pmconf->{'pmexec'}
     # $pmconf->{'listpath'}
     # $pmconf->{'statusfile'}
     # $pmconf->{'feeds'}
     # $pmconf->{'archivepath'}
     # $pmconf->{'archiveurl'}
     # $pmconf->{'feeddefs'} (Array of options, each line)
     #
     my $pmconfigfile = shift;
     my $pmtemplatefile = shift;

     use File::Basename;
     my $pmconf = ();
     my $pmconf_found = ();
     if (-f $pmconfigfile) {
          my @config_data = read_config($pmconfigfile);
          $pmconf = $config_data[0]->{'pmconfig'}[0];
          $pmconf->{'pm'} = fileparse($pmconf->{'pmexec'}) if ($pmconf->{'pm'} eq "");
     }
     if (! -f $pmconfigfile || (defined $pmconf->{'pmexec'} && ! -f $pmconf->{'pmexec'})) {
          my @config_data = read_config($pmtemplatefile);
          foreach my $pkm (keys %{$config_data[0]}) {
               if (-f $config_data[0]{$pkm}{'pmexec'}) {
                    $pmconf_found = $config_data[0]{$pkm};
                    $pmconf_found->{'pm'} = $pkm;
                    last;
               }
          }
          if (scalar(keys(%{$pmconf_found}))) {
               if ($pmconf_found->{'archiveurl'} =~ m/^\/webman.*/i) {
                    my $dsmhostname = lc(`/bin/hostname`);
                    chomp($dsmhostname);
                    my $adminport = `/bin/get_key_value /etc/synoinfo.conf admin_port`;
                    chomp($adminport);
                    $pmconf_found->{'archiveurl'} = 'http://'.$dsmhostname.':'.$adminport.$pmconf_found->{'archiveurl'};
               }
               my $pm_json = JSON::XS->new->encode($pmconf_found);
               my $pm_json = '{"pmconfig":['.$pm_json.']}';
               if (open(PMDAT,">",$pmconfigfile)) {
                    flock(PMDAT, LOCK_EX);
                    print PMDAT $pm_json;
                    close(PMDAT);
               } else {
                    close(DAT);
               }
               $pmconf = $pmconf_found;
          } else {
               $pmconf->{'pm'} = 'wrong' if (defined $pmconf->{'pmexec'});
          }
          unlink ('/var/packages/ipkgui/etc/packages_ref.json') if (-f '/var/packages/ipkgui/etc/packages_ref.json');
          unlink ('/var/packages/ipkgui/etc/sections.json') if (-f '/var/packages/ipkgui/etc/sections.json');
          unlink ('/var/packages/ipkgui/etc/lastchange') if (-f '/var/packages/ipkgui/etc/lastchange');
     }
     return $pmconf;
}
sub read_config {
     my $configfile = shift;

     my $raw_data;
     if (-e $configfile) {
          if (open(DAT,"<".$configfile)) {
               $raw_data = <DAT>;
               close(DAT);
          } else {
               close(DAT);
               return @config_data;
          }
     } else {
          return @config_data;
     }
     my @data = JSON::XS->new->decode($raw_data);
     return @data;
}
sub read_list {
     my $cmd = shift;

     my ($in,$iv,$il,$i);
     $i = 0;
     $empty = 0;
     my @package = ();
     open(HANDLE, $cmd) or die $!;
     while (<HANDLE>) {
          if ($_ eq "\x0A" || $_ =~ /^Successfully terminated/) {
               $empty = 1;
               next;
          } elsif ($empty == 1) {
               $i++;
               $empty = 0;
          }
          $_ =~ tr/\n//d;
          ($p,$v) = split(/: /,$_, 2);
          $package[$i]{lc $p} = trim($v) if (defined $v);
     }
     close(HANDLE);
     return @package;
}
sub get_feeds {
     my $pmconfpath = shift;

     use File::Basename;
     my (@feed_data,$i,$pa,$feedline,%match,@feed_temp_opts,$feed_temp);
     my @feeds = ();
     my @conffiles;
     if (-d $pmconfpath) {
          opendir(DIR,$pmconfpath);
          @conffiles = grep(!/^\./,readdir(DIR));
          closedir(DIR);
          $i = 0;
          foreach $pa (@conffiles) {
               $conffiles[$i] = $pmconfpath."/".$conffiles[$i];
               $i++;
          }
     } elsif (-f $pmconfpath) {
          $conffiles[0] = $pmconfpath;
     }
     foreach $list (@conffiles) {
          @feed_temp_opts = ();
          if (open(FEED,"<".$list)) {
               @feed_data = <FEED>;
               close (FEED);
               chomp(@feed_data);
          } else {
               close(FEED);
          }
          foreach $feedline (@feed_data) {
               @match{"feedactive", "feedtype", "feedname", "feedurl"} = $feedline =~ m/(#?) ?(src\/gz|src) ?([a-z0-9_\-]+) ?((https?):\/\/(www\.)?[a-z0-9\-]+(\.?[a-z0-9\-]+)+([\:0-9]+)?([\/?].*)?)/i;
               if (defined $match{'feedtype'}) {
                    $feed_temp = ();
                    if ($match{'feedactive'} eq "#") {
                         $feed_temp->{'feedactive'} = 0;
                    } else {
                         $feed_temp->{'feedactive'} = 1;
                    }
                    $feed_temp->{'feedtype'} = $match{'feedtype'} if defined $match{'feedtype'};
                    $feed_temp->{'feedname'} = $match{'feedname'} if defined $match{'feedname'};
                    $feed_temp->{'feedurl'} =  $match{'feedurl'} if defined $match{'feedurl'};
                    $feed_temp->{'feedfile'} = fileparse($list);
                    push (@feeds,$feed_temp);
               } else {
                    push (@feed_temp_opts,$feedline);
               }
          }
          foreach my $f (@feeds) {
               push (@{$f->{'feedopts'}},@feed_temp_opts) if ($f->{'feedfile'} eq fileparse($list));
          }
#print Dumper(\@feeds);
#exit;
     }
     return @feeds;
}
sub get_feed_by_package {
     my ($iblistpath,$package,$version,@feeds) = @_;

     my ($i,$empty,@prop,@propline,$fd,$feedactive,$feedname,$feedurl,$fh);
     foreach $fd (@feeds) {
          if ($fd->{'feedactive'}) {
               $i = 0;
               $empty = 0;
               @prop = ();
               @propline = ();
               if (-e $iblistpath."/".$fd->{'feedname'}) {
                    open($fh, "-|", "/bin/zcat -f ".$iblistpath."/".$fd->{'feedname'});
                    if (defined $fh) {
                         while (<$fh>) {
                              if ($_ eq "\x0A") {
                                   $empty = 1;
                                   next;
                              } elsif ($empty == 1) {
                                   $i++;
                                   $empty = 0;
                              }
                              $_ =~ tr/\n//d;
                              ($p,$v) = split(/: /,$_, 2);
                              $prop[$i]{lc $p} = trim($v) if (trim($v));
                         }
                         close($fh);
#print Dumper(@prop);
                         if (@propline = grep {$_->{'package'} eq $package && $_->{'version'} eq $version} @prop) {
                              $feedname = $fd->{'feedname'};
                              $feedurl = $fd->{'feedurl'};
                              last;
                         }
                    }
               }
          }
     }
     return ($feedname,$feedurl,@propline);
}
sub md5hex_file {
     my $fname = shift;
     my $md5file = "";
     if (open FILE, "$fname") {
          my $md5file = Digest::MD5->new;
          $md5file->addfile (*FILE);
          my $hash = $md5file->hexdigest;
          close (FILE);
          return $hash;
     } else {
          return 0;
     }
}
sub sha256hex_file {
     my $fname = shift;

     my $sha256file = "";
     if (open FILE, "$fname") {
          my $sha256file = Digest::SHA->new(256);
          $sha256file->addfile (*FILE);
          my $hash = $sha256file->hexdigest;
          close (FILE);
          return $hash;
     } else {
          return 0;
     }
}
sub read_crontab {
     my ($crontab,$filter) = @_;

     # read crontab and ignore old entry (delete)
     open(CRON,"<".$crontab);
     my @cron_data = grep(!/$filter/, <CRON>);
     close (CRON);
     return @cron_data;
}
sub write_crontab {
     my ($crontab,@cron_data) = @_;

     use File::Copy;
     # write crontab.tmp
     open(CRON,">".$crontab.".tmp");
     print CRON @cron_data;
     close (CRON);
     if (-e $crontab.".tmp") {
          move($crontab.".tmp", $crontab);
     }
     return $?;
}
sub convert_iso2syno {
     my $lang = shift;

     my %iso2syno;
     $iso2syno{"de"} = "ger";  #german
     $iso2syno{"en"} = "enu";  #english US
     $iso2syno{"zh"} = "chs";  #chinese simplified
     $iso2syno{"cs"} = "csy";  #czech
     $iso2syno{"jp"} = "jpn";  #japan
     $iso2syno{"ko"} = "krn";  #korean
     $iso2syno{"da"} = "dan";  #danish
     $iso2syno{"fr"} = "fre";  #french
     $iso2syno{"it"} = "ita";  #italian
     $iso2syno{"nl"} = "nld";  #dutch
     $iso2syno{"no"} = "nor";  #norwegian
     $iso2syno{"pl"} = "plk";  #polish
     $iso2syno{"ru"} = "rus";  #russian
     $iso2syno{"sp"} = "spn";  #spanish
     $iso2syno{"sv"} = "sve";  #swedish
     $iso2syno{"hu"} = "hun";  #hungarian
     $iso2syno{"tr"} = "trk";  #turkish
     $iso2syno{"pt"} = "ptg";  #portuguese european
     return $iso2syno{$lang};
}
sub convert_syno2iso {
     my $lang = shift;

     my %syno2iso;
     $syno2iso{"ger"} = "de";  #german
     $syno2iso{"enu"} = "en";  #english US
     $syno2iso{"chs"} = "zh";  #chinese simplified
     $syno2iso{"csy"} = "cs";  #czech
     $syno2iso{"jpn"} = "jp";  #japan
     $syno2iso{"krn"} = "ko";  #korean
     $syno2iso{"dan"} = "da";  #danish
     $syno2iso{"fre"} = "fr";  #french
     $syno2iso{"ita"} = "it";  #italian
     $syno2iso{"nld"} = "nl";  #dutch
     $syno2iso{"nor"} = "no";  #norwegian
     $syno2iso{"plk"} = "pl";  #polish
     $syno2iso{"rus"} = "ru";  #russian
     $syno2iso{"spn"} = "sp";  #spanish
     $syno2iso{"sve"} = "sv";  #swedish
     $syno2iso{"hun"} = "hu";  #hungarian
     $syno2iso{"trk"} = "tr";  #turkish
     $syno2iso{"ptg"} = "pt";  #portuguese european
     return $syno2iso{$lang};
}
sub get_language {
     my $user = shift;

     my $dsmlanguage = `/bin/get_key_value /etc/synoinfo.conf language`;
     chomp($dsmlanguage);

     my @bl = ($ENV{HTTP_ACCEPT_LANGUAGE} ne "") ? split(',',$ENV{HTTP_ACCEPT_LANGUAGE}) : '';
     my $browserlanguage = (scalar(@bl)) ? convert_iso2syno(substr($bl[0],0,2)) : $dsmlanguage;

     $dsmlanguage = ($dsmlanguage eq "def") ? $browserlanguage : $dsmlanguage;

     my ($raw_data,$json_data,$jdata);
     my $usersettings_file = "/usr/syno/etc/preference/".$user."/usersettings";
     if (-e $usersettings_file) {
          if (open(DAT, $usersettings_file)) {
               $raw_data = <DAT>;
               close(DAT);
               $jdata = JSON::XS->new->utf8;
               $json_data = $jdata->decode($raw_data);
               if (${$json_data}{Personal}->{lang} && ${$json_data}{Personal}->{lang} ne 'def') {
                    $dsmlanguage = ${$json_data}{Personal}->{lang};
               }
          }
     }
     return $dsmlanguage;
}
sub update_appnotify {
     my ($appname,$update_count) = @_;

     my @alluser = all_user('local');
     my $appnotifycmd = "/usr/syno/bin/synoappnotify -c ".$appname." -u ";
     foreach my $u (@alluser) {
          $result = `$appnotifycmd $update_count "$u"`;
     }
}
1;
