#!/usr/bin/perl
#*******************************************************************#
#  iPKGui - 3rdparty ipkg/opkg browser/webUI                        #
#  Modul:       import.cgi                                          #
#  Description: a perl script to upload a file/value and import     #
#  Author:      QTip from the german Synology support forum         #
#  Copyright:   2014-2015 by QTip                                   #
#  License:     GNU GPLv3 (see LICENSE)                             #
#  ---------------------------------------------------------------  #
#  Version:     0.16 - 21/10/2015                                   #
#  for more information check the changelog                         #
#*******************************************************************#

$user = `/usr/syno/synoman/webman/modules/authenticate.cgi`;
$user =~ s/^\s+|\s+$//g;
exit if ($user eq '');

use strict;
use warnings;
use lib ('/var/packages/ipkgui/target/include','/var/packages/ipkgui/target/lib');

use CGI;
use CGI::Carp qw(fatalsToBrowser);
use Log::Tiny;
use Fcntl qw(:flock);
use POSIX qw(strftime tzset);

sub upload_file{
     my ($filename,$filehandle,$path) = @_;
     my $target = $path."/".$filename;
     binmode $filehandle;
     open(TARGET,">$target") or die $!;
     binmode TARGET;
     my $buffer;
     while(read $filehandle,$buffer,1024){
          print TARGET $buffer;
     }
     close TARGET;
     return ($? > 255) ? ($? % 256) : $?;
}

my $program = "ipkgui";
my $homepath = "/var/packages/$program/target";
my $importpath = "$homepath/tmp";
my $logfile = "$homepath/log/$program.log";
my $max_filesize = '32M';
$max_filesize = 1024 * 1024 * substr($max_filesize,0,-1);
unless (-d $importpath) {mkdir($importpath,0777) || die "Could not create import dir ($!)\n\n";}
unless (-d "$homepath/log") {mkdir($homepath."/log",0777) || die "Could not create log dir ($!)\n\n";}
my $log = Log::Tiny->new($logfile) || die "Could not create logfile! (".Log::Tiny->errstr.")\n\n";
my $tsf = strftime "%Y-%m-%d %H:%M:%S", localtime;
$log->format("$tsf ipkgui[%p]: %c: %m\n");

my $dsmmajor = `/bin/get_key_value /etc.defaults/VERSION majorversion`;
chomp($dsmmajor);

my ($filehandle,$filename,$filesize,$result,$ret,$upload_file);

if ($ENV{'REQUEST_METHOD'} eq "POST") {
     my $cgi = CGI->new();
     my %params = $cgi->Vars();
     if ($params{'action'} eq "import" && $params{'file'} ne "") {
          print "Content-type: application/json; charset=UTF-8\n\n";
          $upload_file = $importpath."/ipkgui_import";
          if (-e $upload_file) {
               if (open(DAT, $upload_file)) {
                    flock(DAT, LOCK_EX);
                    $result = <DAT>;
                    close(DAT);
               } else {
                    close(DAT);
                    print '{"success":false, "msg":IPKG.Browser.Texts[\'StatusImportFailedText\']}';
                    exit;
               }
               $result =~ tr/+/ /;
               $result =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
               # prevent include of server side includes
               $result =~ s/<!--(.|\n)*-->//g;
               $result =~ s/[^a-zA-Z0-9_\-\{\}\[\]\:\,\"]|\n|\r//g;
               $result =~ s/\s+/ /g;
               print $result;
               $log->INFO("\'$params{'file'}\' successfully imported!");
          } else {
               print '{"success":false, "msg":IPKG.Browser.Texts[\'StatusImportFailedText\']}';
               $log->ERROR("There was an error during import of the file \'$params{'file'}\'!");
          }
     } elsif ($params{'action'} eq "upload" && $params{'ibImpExpImportFile'} ne "") {
          print $cgi->header(-type => 'text/html');
#print $params{'action'};
          $filehandle = $cgi->upload('ibImpExpImportFile');
          $filename = 'ipkgui_import';
          $filesize = -s $filehandle;
          if ($max_filesize == 0 || ($max_filesize > 0 && $filesize > $max_filesize)) {
               print '{"success":false, "msg":IPKG.Browser.Texts[\'StatusImportSizeText\']+" "+'.($max_filesize/1024).'+" kb"}';
               exit;
          }
          print $cgi->start_html();
          $ret = upload_file($filename,$filehandle,$importpath);
          print $cgi->end_html();
          if ($ret == 0) {
               print '{"success":true}';
          } else {
               print '{"success":false, "msg":IPKG.Browser.Texts[\'StatusImportFailedText\']}';
               $log->ERROR("There was an error during upload of the file \'$params{'file'}\'!");
          }
     }
}
exit;
