#!/usr/bin/perl
#*******************************************************************#
#  iPKGui - 3rdparty ipkg/opkg browser/webUI                        #
#  Modul:       package_read.pl                                     #
#  Description: a perl script to get the list of ipkg/opkg/dpkg     #
#               packages                                            #
#  Author:      QTip from the german Synology support forum         #
#  Copyright:   2012-2014 by QTip                                   #
#  License:     GNU GPLv3 (see LICENSE)                             #
#  ---------------------------------------------------------------  #
#  Version:     0.6 - 25/05/2014                                    #
#  for more information check the changelog                         #
#*******************************************************************#

sub get_packages {
     my ($pmstatcmd,$pminfocmd,$iblistpath) = @_;

     my (@instline,$il,$i,$item);
     $il = "";
     my @packstat = ();
     my @package = ();
     my @pckg = ();
     my @sections = ();

     # generate list of installed packages and create a array
     @packstat = read_list($pmstatcmd);

     # read list of all packages in a array of hashes
     @pckg = read_list($pminfocmd);
     foreach $item (@pckg) {
          # eleminate unnecessary entrys
          if (defined $item->{'description'}) {
               push (@package,$item);
          }
     }
     my $package_count = scalar(@package);

     # if archive available
     if (-e $iblistpath."/archive") {
          @archive = read_list($iblistpath."/archive");
          if ($package_count == 0) {
               for (@archive) {
                   my %copy_hash = %$_;
                   push @package, \%copy_hash;
               }
          }
          foreach $item (@archive) {
               $item->{'status'} = 'available_archive';
          }
          push (@package,@archive);
     }
     @package = sort{$a->{'package'} cmp $b->{'package'} || format_version($b->{'version'}) cmp format_version($a->{'version'})} @package;

     # collect all versions from packages
     $i = 0;
     my (@clean,%pversion);
     foreach $item (@package) {
          if (!grep {$_->{'package'} eq $item->{'package'} && $_->{'status'} ne "available_archive"} @clean) {
               push(@clean,$item)
          } elsif ($item->{'status'} eq "available_archive") {
               push(@clean,$item)
          }
          if (@dupline = grep {$_->{'package'} eq $item->{'package'}} @package) {
               if (scalar(@dupline) > 1) {
                    push (@{$pversion{$item->{'package'}}},$item->{'version'}) if (!in_array($item->{'version'},@{$pversion{$item->{'package'}}}));
               }
          }
     }
     foreach $item (keys %pversion) {
          delete $pversion{$item} if (scalar(@{$pversion{$item}}) == 1);
     }

     # compare installed and available packages, set status and installed version
     foreach $item (@clean) {
          if ($item->{'status'} ne "available_archive") {
               if (@instline = grep {$_->{'package'} eq $item->{'package'}} @packstat) {
                    if (grep /unknown[ ]hold[ ]installed/, $instline[0]{'status'}) {
                         $item->{'iversion'} = $instline[0]{'version'};
                         $item->{'status'} = "installed_extern";
                    } elsif (grep /[ ]hold/, $instline[0]{'status'}) {
                         $item->{'iversion'} = $instline[0]{'version'};
                         $item->{'status'} = "installed_hold";
                    } else {
                         if (grep {$_ eq $item->{'package'}} %pversion) {
                              foreach my $aver (@{$pversion{$item->{'package'}}}) {
                                   if (format_version($aver) gt format_version($instline[0]{'version'})) {
                                        $item->{'iversion'} = $instline[0]{'version'};
                                        $item->{'status'} = "installed_upgradeable";
                                        last;
                                   } elsif (format_version($aver) le format_version($instline[0]{'version'})) {
                                        $item->{'iversion'} = $instline[0]{'version'};
                                        $item->{'status'} = "installed_updated";
                                        last;
                                   }
                              }
                         } else {
                              if (format_version($item->{'version'}) gt format_version($instline[0]{'version'})) {
                                   $item->{'iversion'} = $instline[0]{'version'};
                                   $item->{'status'} = "installed_upgradeable";
                              } elsif (format_version($item->{'version'}) le format_version($instline[0]{'version'})) {
                                   $item->{'iversion'} = $instline[0]{'version'};
                                   $item->{'status'} = "installed_updated";
                              }
                         }
                    }
               } else {
                    $item->{'iversion'} = "";
                    $item->{'status'} = "available";
               }

               # delete not needed fields
               delete $item->{'maintainer'};
               delete $item->{'md5sum'};
               delete $item->{'filename'};
               delete $item->{'source'};
               delete $item->{'architecture'};

               # group nearly identical section entrys
               $item->{'section'} = lc($item->{'section'});
               $item->{'section'} =~ s/(-|\/)/_/g;
               my $search_package = substr($item->{'section'},0,5);
               my $found_section = 0;
               for (0..$#sections) {
                    if ($sections[$_] =~ /^$search_package/) {
                         $item->{'section'} = $sections[$_];
                         last;
                    }
               }
               if ($item->{'description'}) {
                    $item->{'description'} =~ s/\\"/"/g;
               }
               if ($item->{'depends'}) {
                    $item->{'depends'} =~ s/, /,/g;
                    $item->{'depends'} =~ s/ \(/\(/g;
                    $item->{'depends'} =~ s/([a-zA-Z0-9\-\+])[ ]([a-zA-Z0-9\-\+])/$1,$2/;
               }
               if ($item->{'suggests'}) {
                    $item->{'suggests'} =~ s/, /,/g;
                    $item->{'suggests'} =~ s/ \(/\(/g;
                    $item->{'suggests'} =~ s/([a-zA-Z0-9\-\+])[ ]([a-zA-Z0-9\-\+])/$1,$2/;
               }
               if ($item->{'conflicts'}) {
                    $item->{'conflicts'} =~ s/, /,/g;
                    $item->{'conflicts'} =~ s/ \(/\(/g;
                    $item->{'conflicts'} =~ s/([a-zA-Z0-9\-\+])[ ]([a-zA-Z0-9\-\+])/$1,$2/;
               }
          }
     }

     # group double packages by version
     $i = 0;
     my $pv;
     foreach $item (@clean) {
          if (grep {$_ eq $item->{'package'} && $item->{'status'} ne "available_archive"} %pversion) {
               if (!is_array($clean[$i]->{'version'}) && scalar(@{$pversion{$item->{'package'}}}) > 1) {
                    $clean[$i]->{'version'} = [];
               }
               push @{$clean[$i]->{'version'}},@{$pversion{$item->{'package'}}};
          }
          $i++;
     }
     return @clean;
}

sub format_version {
     my $val = shift;

     my (@f,@r,$n);
     if ($val) {
          @f = split(/\./,$val);
          for (0..$#f) {
               $f[$_] = sprintf("%03s", $f[$_]);
               $f[$_] =~ tr/ /0/;
          };
          @r = split('-',$f[($#f)]);
          if (@r) {
               $r[0] = sprintf("%03s", $r[0]);
               $r[0] =~ tr/ /0/;
               $f[($#f)] = join('-',@r);
          }
          $n = join('.',@f);
          return $n;
     } else {
          return '';
     }
}
1;
