#!/usr/bin/perl
#*******************************************************************#
#  iPKGui - 3rdparty ipkg/opkg browser/webUI                        #
#  Modul:       read_packages.cgi                                   #
#  Description: a perl script to get the data from packages         #
#               list                                                #
#  Author:      QTip from the german Synology support forum         #
#  Copyright:   2012-2015 by QTip                                   #
#  License:     GNU GPLv3 (see LICENSE)                             #
#  ---------------------------------------------------------------  #
#  Version:     0.65 - 21/07/2015                                   #
#  for more information check the changelog                         #
#*******************************************************************#

$user = `/usr/syno/synoman/webman/modules/authenticate.cgi`;
$user =~ s/^\s+|\s+$//g;
exit if ($user eq '');

use strict;
use warnings;
use lib ('/var/packages/ipkgui/target/include','/var/packages/ipkgui/target/lib');

use CGI;
use CGI::Carp qw(fatalsToBrowser);
use File::stat;
use JSON::XS;
#use Data::Dumper;

## initialize vars #################
my (@parts,$part,$buffer,$value,$name,%param);
my ($pack_data,$pack_data_full,$ret,$raw_data,%match,$lastchange);
my @packread = ();
my $packagehome = "/var/packages/ipkgui";
my $configpath = $packagehome."/etc/";
my $pmconfigfile = $configpath."pmconfig.json";
my $pmtemplatefile = "$packagehome/target/extras/pmtemplates.json";
my $changesfile = "$packagehome/etc/lastchange";
my $changestamp = 0;

require "functions.pl";

###################################

print "Content-type: application/json; charset=UTF-8\n\n";

if ($ENV{'REQUEST_METHOD'} eq "POST") {
     read(STDIN,$buffer,$ENV{"CONTENT_LENGTH"});
     @parts = split(/\&/,$buffer);
     foreach $part (@parts) {
          ($name,$value) = split(/=/,$part);
          $value =~ tr/+/ /;
          $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
          # prevent include of server side includes
          $value =~ s/<!--(.|\n)*-->//g;
          $param{$name} = $value;
     }
     my $pmconfig = detect_pkm($pmconfigfile,$pmtemplatefile);
     if (scalar(keys(%{$pmconfig})) == 0 && $param{'packtype'} ne "smallfixes") {
          print '({"success":false,"data":[],"msg":"StatusNoIPKGText"})';
          exit;
     }
     my $pmupdcmd = "$pmconfig->{'pmexec'} update 2> /dev/null";
     my $pminfocmd = "$pmconfig->{'pmexec'} info 2> /dev/null|";
     my $pmstatcmd = "$pmconfig->{'pmexec'} status 2> /dev/null|";
     $lastchange = (-e $pmconfig->{'statusfile'} ? stat($pmconfig->{'statusfile'})->mtime : 0);
     if (-e $changesfile) {
          $changestamp = `cat $changesfile >&1`;
     }
     if (! -e $changesfile || $lastchange > $changestamp) {
          if (open(TIM,">".$changesfile)) {
               print TIM $lastchange;
               close (TIM);
          } else {
               close(TIM);
          }
     }
     if ($param{'action'} eq "check") {
          detect_pkm($pmconfigfile,$pmtemplatefile);
     } elsif ($param{'action'} eq "read" || $param{'action'} eq "update") {
          if ($param{'action'} eq "update") {
               # update list of packages
               $ret = `$pmupdcmd`;
               if ($? != 0) {
                    print '({"success":false,"data":[],"msg":"StatusIPKGUpdateFailedText"})';
                    exit;
               }
          }
          if ($param{'action'} eq "update" || ! -e $packagehome."/etc/sections.json" || ! -e $packagehome."/etc/packages_ref.json") {
               require "package_read.pl";
               require "package_update.pl";
               @packread = get_package_updates($pmstatcmd,$pminfocmd,$packagehome,$pmconfig->{'listpath'});
          } elsif ($param{'action'} eq "read") {
               if ($param{'mode'} eq "full" || $lastchange > $changestamp) {
                    require "package_read.pl";
                    @packread = get_packages($pmstatcmd,$pminfocmd,$pmconfig->{'listpath'});
                    $pack_data_full = JSON::XS->new->utf8->encode(\@packread);
               } else {
                    if (-e $packagehome."/etc/packages_ref.json") {
                         if (open(DAT,"<".$packagehome."/etc/packages_ref.json")) {
                              $pack_data = <DAT>;
                              @packread = @{JSON::XS->new->utf8->decode($pack_data)};
                              close (DAT);
                         } else {
                              close(DAT);
                         }
                    }
               }
          }
          if ($param{'packtype'} eq "smallfixes") {
               require "package_read.pl";
               @packread = get_packages('/var/lib/dpkg/status','/var/lib/dpkg/available','');
          }
          $pack_data = JSON::XS->new->utf8->encode(\@packread);
          print '{"data":'.$pack_data.'}';
          if ($param{'mode'} eq "full") {
               # write changes in mode 'full' into list of packages
               if (open(DAT,">".$packagehome."/etc/packages_ref.json")) {
                    print DAT $pack_data_full;
                    close (DAT);
               } else {
                    close(DAT);
               }
          }
     }
}
