#!/usr/bin/perl
#*******************************************************************#
#  iPKGui - 3rdparty ipkg/opkg browser/webUI                        #
#  Modul:       index.cgi                                           #
#  Description: perl start script of ipkgui                         #
#  Author:      QTip from the german Synology support forum         #
#  Copyright:   2012-2015 by QTip                                   #
#  License:     GNU GPLv3 (see LICENSE)                             #
#  ---------------------------------------------------------------  #
#  Version:     0.56 - 21/10/2015                                   #
#  for more information check the changelog                         #
#*******************************************************************#
# for perl via ipkg/opkg the following packages must also be        #
# installed:                                                        #
#    - perl-json-xs                                                 #
#####################################################################

use strict;
use warnings;
use CGI;
use CGI::Carp qw(fatalsToBrowser);
use lib('/var/packages/ipkgui/target/include');

require "include/check_appprivilege.pl";
my ($synotoken,$synouser,$is_admin) = check_privilege('SYNO.SDS._ThirdParty.App.ipkgui');
# if it return a empty string (not logged in), then exit app
exit if ($synouser eq '');

use File::Path;
use Cwd qw(realpath);
use Data::Dumper;

require "functions.pl";

my $dsm = '';
my $pmconfigfile = "/var/packages/ipkgui/etc/pmconfig.json";
my $pmtemplatefile = "/var/packages/ipkgui/target/extras/pmtemplates.json";
my $smallfixnumber = `/bin/get_key_value /etc.defaults/VERSION smallfixnumber`;
chomp($smallfixnumber);
my $dsmmajor = `/bin/get_key_value /etc.defaults/VERSION majorversion`;
chomp($dsmmajor);
$dsm = '5' if ($dsmmajor >= 5);
my $ibmajor = `/bin/get_key_value /var/packages/ipkgui/target/VERSION MAJOR`;
chomp($ibmajor);
my $ibminor = `/bin/get_key_value /var/packages/ipkgui/target/VERSION MINOR`;
chomp($ibminor);
my $ibbuild = `/bin/get_key_value /var/packages/ipkgui/target/VERSION BUILD`;
chomp($ibbuild);
my $dsmlanguage = get_language($synouser);

unless (-e '/usr/syno/synoman/webman/3rdparty/ipkgui/texts/'.$dsmlanguage.'/gui.js') {
     $dsmlanguage = "enu";
}
opendir(DIR,"/usr/syno/synoman/webman/3rdparty/ipkgui/texts");
my @dirs = grep(!/^\./,readdir(DIR));
closedir(DIR);
my $iblanguages = join(" ",sort(@dirs));

my $pmconfig = detect_pkm($pmconfigfile,$pmtemplatefile);
my $ibarchivepath = $pmconfig->{'archivepath'};
my $ibarchiveurl = $pmconfig->{'archiveurl'};
if ($ibarchiveurl =~ m/(.*)(webman.*)/i) {
     $ibarchiveurl =~ s/^(.*)(webman.*)/\/usr\/syno\/synoman\/$2/i;
} else {
     $ibarchiveurl =~ s/^((https?):\/\/(www\.)?[a-z0-9\-:@]+(\.?[a-z0-9\-:@]+)+([\:0-9]+)?([\/?].*)?)/\/var\/services\/web$6/i;
}
my $pkg_tp = '/usr/syno/synoman/webman/3rdparty/ipkgui';
my $pkg_vol = realpath($pkg_tp);
my $rest_url = substr($ibarchiveurl,length($pkg_tp));
my $ibarchiveurl_real = $pkg_vol.$rest_url;
$ibarchivepath = realpath($ibarchivepath);
mkpath($ibarchivepath) if (! -e $ibarchivepath);
if ($ibarchivepath ne $ibarchiveurl_real && ! -e $ibarchiveurl_real) {
     symlink ($ibarchivepath,$ibarchiveurl_real);
}

if (-d '/var/packages/ipkgui/target/tmp/') {
     unlink glob('/var/packages/ipkgui/target/tmp/*');
}
if (-e '/var/packages/ipkgui/etc/new.dat') {
     unlink ('/var/packages/ipkgui/etc/new.dat');
}

print "Content-type: text/html; charset=UTF-8\n\n";

print <<EOTBEGIN;
<!DOCTYPE html>
<html>
<head>
     <meta http-equiv="X-UA-Compatible" content="IE=9">
     <meta http-equiv="pragma" content="no-cache"><meta http-equiv="expires" content="-1"><meta http-equiv="cache-control" content="no-cache">

EOTBEGIN
     print "<link rel='stylesheet' type='text/css' href='extjs/resources/css/ext-all.css?v=".$ibbuild."'>";
     print "<link rel='stylesheet' type='text/css' href='extjs/resources/css/xtheme-dsm".$dsm.".css?v=".$ibbuild."'>";
     print "<link rel='stylesheet' type='text/css' href='scripts/statusbar".$dsm.".css?v=".$ibbuild."'>";
     print "<link rel='stylesheet' type='text/css' href='scripts/fileuploadfield.css?v=".$ibbuild."'>";
     print "<link rel='stylesheet' type='text/css' href='scripts/ipkgui".$dsm.".css?v=".$ibbuild."'>";

print <<EOTBEGIN;
     <link rel="shortcut icon" href="favicon.ico">
     <script type="text/javascript" src="extjs/adapter/ext/ext-base.js?v=$ibbuild"></script>
     <script type="text/javascript" src="extjs/ext-all.js?v=$ibbuild"></script>
     <script type="text/javascript" src="extjs/ext-fix.js?v=$ibbuild"></script>
     <script type="text/javascript" src="scripts/CheckColumn.js?v=$ibbuild"></script>
     <script type="text/javascript" src="scripts/StatusBar.js?v=$ibbuild"></script>
     <script type="text/javascript" src="scripts/general_functions.js?v=$ibbuild"></script>
     <script type="text/javascript" src="scripts/FileUploadField.js?v=$ibbuild"></script>
     <script language="javascript" type="text/javascript">
          Ext.ns('IPKG.Browser');
          IPKG.Browser.languages = '$iblanguages';
          IPKG.Browser.current_user = '$synouser';
          IPKG.Browser.packtype = '';
          IPKG.Browser.pkgm = '$pmconfig->{'pm'}';
          IPKG.Browser.synotoken = '$synotoken';
          IPKG.Browser.dsmmajor = '$dsmmajor';
          IPKG.Browser.smallfixnumber = '$smallfixnumber';
          IPKG.Browser.spkver = '$ibmajor.$ibminor-$ibbuild';
     </script>
EOTBEGIN

     print "<script type='text/javascript' src='extjs/locale/ext-lang-".convert_syno2iso($dsmlanguage).".js?v=".$ibbuild."'></script>";
     print "<script type='text/javascript' src='texts/$dsmlanguage/gui.js?v=".$ibbuild."'></script>";
     print "<script type='text/javascript' src='scripts/functions.js?v=".$ibbuild."'></script>";
     print "<script type='text/javascript' src='scripts/browser.js?v=".$ibbuild."'></script>";
     print "<script type='text/javascript' src='scripts/ipkgui.js?v=".$ibbuild."'></script>";

print <<EOTEND;
</head><title>iPKGui</title><body>
<script language="javascript" type="text/javascript">
document.getElementsByTagName("body")[0].style.cursor = 'auto';
</script></body></html>

EOTEND
